#include "DataProvider.h"
#include "SPLE_lib.h"

extern "C" {

typedef Matrix<double,Dynamic,Dynamic,RowMajor> RMatrixXd;
typedef Matrix<int,Dynamic,Dynamic,RowMajor> RMatrixXi;

int * classify(
		double * noisy
,		int image_rows
,		int image_cols
,		double sigma
,		int patch_size
,		int num_iterations
){
	int overlap = patch_size - 1;
	int data_size = patch_size * patch_size;
	int std_num_factors = 0.5 * data_size;
	int num_orientations = 18;
	int num_models = num_orientations + 2;
	MatrixXd noisy_image = Map<RMatrixXd>(noisy, image_rows, image_cols);
	imwrite("noisy.png", &noisy_image, 1);
	int map_rows = num_patches(image_rows, patch_size, overlap);
	int map_cols = num_patches(image_cols, patch_size, overlap);
	MatrixXi patch_map(map_rows, map_cols);

	VectorXd * prior = NULL;
	MatrixXd * factors = NULL;
	VectorXd * mus = NULL;
	VectorXd ** responsibility = NULL;
	read_config(data_size, std_num_factors, num_models, prior, mus, factors);

	VectorXd ** noisy_patches = image2patches(noisy_image, image_rows, image_cols, overlap, patch_size);
	for(int iter = 0; iter < num_iterations; iter++){
		calc_responsibilities(noisy_patches, factors, mus, prior, map_rows, map_cols, data_size, std_num_factors, sigma, responsibility, patch_map);
		if(iter < num_iterations - 1)
			update_PPCA(noisy_patches, responsibility, prior, num_models, map_rows, map_cols, sigma, mus, factors);
	}

	print_patch_map(patch_map);

	// prepare for python to read in
	RMatrixXi out_map = patch_map;
	int * classified = new int [map_rows*map_cols];
	for(int k = 0; k < map_rows*map_cols; k++)
		classified[k] = out_map.data()[k];

	for( int row = 0; row < map_rows; row++ ){
		delete [] noisy_patches[row];
		delete [] responsibility[row];
	}
	delete [] noisy_patches;
	delete [] responsibility;
	delete [] factors;
	delete [] mus;
	delete [] prior;

	return classified;
}
}
